require 'json'
require 'logger'
require 'net/http'
require 'i18n'

require 'sms_centre/version'

I18n.load_path += Dir[File.expand_path('..', __FILE__) + '/sms_centre/locales/*.yml']

module SMSCentre
  ERRORS = {
    (ERROR_PARAMS            = 1) => 'error.parameters',
    (ERROR_AUTH              = 2) => 'error.auth',
    (ERROR_BALANCE           = 3) => 'error.balance',
    (ERROR_FLOOD             = 4) => 'error.flood',
    (ERROR_DATE_FORMAT       = 5) => 'error.data_format',
    (ERROR_FORBIDDEN_MESSAGE = 6) => 'error.forbidden_message',
    (ERROR_PHONE_FORMAT      = 7) => 'error.phone_format',
    (ERROR_IMPOSSIBLE        = 8) => 'error.impossible',
    (ERROR_REPEATED_REQUEST  = 9) => 'error.repeated_request',
  }

  MESSAGE_STATUSES = {
    (MESSAGE_STATUS_PENDING                 = -1) => 'message_status.pending',
    (MESSAGE_STATUS_OPERATOR                = 0 ) => 'message_status.operator',
    (MESSAGE_STATUS_DELIVERED               = 1 ) => 'message_status.delivered',
    (MESSAGE_STATUS_EXPIRED                 = 3 ) => 'message_status.expired',
    (MESSAGE_STATUS_ERROR_IMPOSSIBLE        = 20) => 'message_status.impossible',
    (MESSAGE_STATUS_ERROR_PHONE_FORMAT      = 22) => 'message_status.phone_format_error',
    (MESSAGE_STATUS_ERROR_FORBIDDEN_MESSAGE = 23) => 'message_status.forbidden_message_error',
    (MESSAGE_STATUS_ERROR_BALANCE           = 24) => 'message_status.balance_error',
    (MESSAGE_STATUS_UNAVAILABLE             = 25) => 'message_status.unavailable'
  }

  STATUS_ERRORS = {
    (STATUS_ERROR_PARAMS            = 1) => 'status_error.parameters',
    (STATUS_ERROR_AUTH              = 2) => 'status_error.auth',
    (STATUS_ERROR_NOT_FOUND         = 3) => 'status_error.not_found',
    (STATUS_ERROR_IP_BANNED         = 4) => 'status_error.ip_banned',
    (STATUS_ERROR_REPEATED_REQUEST  = 9) => 'status_error.repeated_request'
  }

  autoload :API,           'sms_centre/api'
  autoload :Configuration, 'sms_centre/configuration'
  autoload :Provider,      'sms_centre/provider'
  autoload :Result,        'sms_centre/result'
  autoload :Status,        'sms_centre/status'

  Error           = Class.new(StandardError)
  ConnectionError = Class.new(Error)
  ServerError     = Class.new(Error)

  class ClientError < Error
    attr_reader :error_code

    def initialize(error)
      if Fixnum === error
        @error_code = error
        super(ERRORS[error])
      else
        super(error)
      end
    end

    def human_message
      if @error_code
        SMSCentre.human_error(@error_code)
      else
        message
      end
    end
  end

  class << self
    attr_writer :configuration

    def configuration
      @configuration ||= Configuration.new
    end

    def reset
      @configuration = Configuration.new
    end

    def configure
      yield(configuration)
    end

    def logger
      configuration.logger
    end

    def human_error(error_code)
      I18n.t('sms_centre.' + ERRORS[error_code])
    end

    def human_message_status(message_status_code)
      I18n.t('sms_centre.' + MESSAGE_STATUSES[message_status_code])
    end

    def human_status_error(status_error_code)
      I18n.t('sms_centre.' + STATUS_ERRORS[status_error_code])
    end
  end

end
